// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund.Components.Climate;

interface

uses Esmf;

type
  /// <summary>State for the climate component</summary>
  IClimateState = public interface

    /// <summary>co2 emissions</summary>
    property mco2: IParameter1Dimensional<Timestep, double> read;
        
    /// <summary>Carbon box 1</summary>
    property cbox1: IVariable1Dimensional<Timestep, double> read;
    
    /// <summary>Carbon box 2</summary>
    property cbox2: IVariable1Dimensional<Timestep, double> read;

    /// <summary>Carbon box 3</summary>
    property cbox3: IVariable1Dimensional<Timestep, double> read;

    /// <summary>Carbon box 4</summary>
    property cbox4: IVariable1Dimensional<Timestep, double> read;

    /// <summary>Carbon box 5</summary>
    property cbox5: IVariable1Dimensional<Timestep, double> read;
    
    /// <summary>Carbon decay in box 1</summary>
    property co2decay1: double read;

    /// <summary>Carbon decay in box 2</summary>
    property co2decay2: double read;

    /// <summary>Carbon decay in box 3</summary>
    property co2decay3: double read;

    /// <summary>Carbon decay in box 4</summary>
    property co2decay4: double read;

    /// <summary>Carbon decay in box 5</summary>
    property co2decay5: double read;

    /// <summary>Fraction of carbon emission in box 1</summary>  
    property co2frac1: double read;  

    /// <summary>Fraction of carbon emission in box 2</summary>  
    property co2frac2: double read;  

    /// <summary>Fraction of carbon emission in box 3</summary>  
    property co2frac3: double read;  

    /// <summary>Fraction of carbon emission in box 4</summary>  
    property co2frac4: double read;  

    /// <summary>Fraction of carbon emission in box 5</summary>  
    property co2frac5: double read;  
    
    /// <summary>Atmospheric CO2 concentration</summary>
    property acco2: IVariable1Dimensional<Timestep, double> read;    
    
    /// <summary> CO2 pre industrial</summary>
    property co2pre: double read;    
     
    /// <summary>Global CH4 emissions</summary>
    property globch4: IParameter1Dimensional<Timestep, double> read;
    
    /// <summary>Atmospheric CH4 concentration</summary>
    property acch4: IVariable1Dimensional<Timestep, double> read;
    
    /// <summary>CH4 decay</summary>
    property ch4decay: double read;
    
    /// <summary> CH4 pre industrial</summary>
    property ch4pre: double read;
    
    /// <summary>Global N2O emissions</summary>
    property globn2o: IParameter1Dimensional<Timestep, double> read;
    
    /// <summary>Atmospheric N2O concentration</summary>
    property acn2o: IVariable1Dimensional<Timestep, double> read;
    
    /// <summary>N2O decay</summary>
    property n2odecay: double read;
    
    /// <summary>N2o pre industrial</summary>
    property n2opre: double read;

    /// <summary> Global SF6 emissions</summary>
    property globsf6: IParameter1Dimensional<Timestep, double> read;

    /// <summary>Atmospheric SF6 concentrations</summary>
    property acsf6: IVariable1Dimensional<Timestep, double> read;

    /// <summary>SF6 pre industrial</summary>
    property sf6pre: double read;

    /// <summary>SF6 decay</summary>
    property sf6decay: double read;
    
    /// <summary>Global SO2 emissions</summary>
    property globso2: IParameter1Dimensional<Timestep, double> read;
    
    /// <summary>Atmospheric SO2 concentration</summary>
    property acso2: IVariable1Dimensional<Timestep, double> read;

    /// <summary>No idea</summary>
    property so2dir: Double read;

    /// <summary> no idea</summary>
    property so2ind: double read;
    
    /// <summary>Radiative forcing from CO2</summary>
    property rfCO2: IVariable1Dimensional<Timestep, double> read;

    /// <summary>Radiative forcing from CH4</summary>
    property rfCH4: IVariable1Dimensional<Timestep, double> read;
    
    /// <summary>Radiative forcing from N2O</summary>
    property rfN2O: IVariable1Dimensional<Timestep, double> read;
    
    /// <summary>Radiative forcing from N2O</summary>
    property rfSF6: IVariable1Dimensional<Timestep, double> read;
    
    /// <summary>Radiative forcing from SO2</summary>
    property rfSO2: IVariable1Dimensional<Timestep, double> read;
    
    /// <summary>Total radiative forcing</summary>
    property radforc: IVariable1Dimensional<Timestep, double> read;

    /// <summary> Maximal radiative forcing</summary>
    property maxrf: Double read write;
    
    /// <summary>Average global temperature</summary>
    property temp: IVariable1Dimensional<Timestep, double> read;
    
    /// <summary>delaytemp</summary>
    property delaytemp: double read;
    
    /// <summary>temps</summary>
    property temps: double read;
    
    property addmp: boolean read;    
    property addco2: IParameter1Dimensional<Timestep, double> read;
    property addch4: IParameter1Dimensional<Timestep, double> read;
    property addn2o: IParameter1Dimensional<Timestep, double> read;
    property addsf6: IParameter1Dimensional<Timestep, double> read;
    
    property ch4add: double read;
    property n2oadd: double read;
    property sf6add: double read;    
    property RunAwayCH4: IParameter1Dimensional<Timestep, double> read;
  end;
  
implementation

end.