// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund.Components.ClimateRegional;

interface

uses Esmf,
  Fund.CommonDimensions;

type

  IClimateRegionalState = public interface
    property temp: IParameter1Dimensional<Timestep, Double> read;
    property bregtmp: IParameter1Dimensional<Region, Double> read;

    property regtmp: IVariable2Dimensional<Timestep, Region, Double> read;
  end;

  ClimateRegionalComponent = public class
  public
    method Run(clock: Clock; state: IClimateRegionalState);
  end;
  
implementation

method ClimateRegionalComponent.Run(clock: Clock; state: IClimateRegionalState);
begin
  var s:=state;
  var t:=clock.Current;

  for each r: Region in Enum.GetValues(typeof(Region)) do
  begin  
    s.regtmp[t,r]:=s.temp[t]*s.bregtmp[r];
  end;
end;

end.