// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund.Components.ImpactDiarrhoea;

interface

uses Esmf,
     Fund.CommonDimensions;

type

  IImpactDiarrhoeaState = public interface
    property diadead: IVariable2Dimensional<Timestep, Region, Double> read;
    property diasick: IVariable2Dimensional<Timestep, Region, Double> read;

    //property diadeadcost: IVariable2Dimensional<Timestep, Region, Double> read;
    //property diasickcost: IVariable2Dimensional<Timestep, Region, Double> read;

    property diamort: IParameter1Dimensional<Region, Double> read;
    property diamortel: Double read;
    property diamortnl: Double read;

    property diayld: IParameter1Dimensional<Region, Double> read;
    property diayldel: Double read;
    property diayldnl: Double read;

    property income: IParameter2Dimensional<Timestep, Region, Double> read;
    property population: IParameter2Dimensional<Timestep, Region, Double> read;
    property gdp90: IParameter1Dimensional<Region, Double> read;
    property pop90: IParameter1Dimensional<Region, Double> read;

    property temp90: IParameter1Dimensional<Region, Double> read;
    property bregtmp: IParameter1Dimensional<Region, Double> read;
    property regtmp: IParameter2Dimensional<Timestep, Region, Double> read;
  end;

  ImpactDiarrhoeaComponent = public class
  public
    method Run(clock: Clock; state: IImpactDiarrhoeaState);        
  end;
  
implementation

method ImpactDiarrhoeaComponent.Run(clock: Clock; state: IImpactDiarrhoeaState);
begin
  var s:=state;
  var t:=clock.Current;

  for each r: Region in Enum.GetValues(typeof(Region)) do
  begin  
    var ypc:=1000.0*s.income[t,r]/s.population[t,r];
    var ypc90:=1000.0*s.gdp90[r]/s.pop90[r];

    // 0.49 is the increase in global temperature from pre-industrial to 1990
    var absoluteRegionalTempPreIndustrial:=s.temp90[r]-0.49*s.bregtmp[r];

    if absoluteRegionalTempPreIndustrial > 0.0 then
    begin
      s.diadead[t,r]:=s.diamort[r]*s.population[t,r]*Math.Pow(ypc/ypc90,s.diamortel)
          *(Math.Pow((absoluteRegionalTempPreIndustrial+s.regtmp[t,r])/absoluteRegionalTempPreIndustrial,s.diamortnl)-1.0);

      s.diasick[t,r]:=s.diayld[r]*s.population[t,r]*Math.Pow(ypc/ypc90,s.diayldel)
          *(Math.Pow((absoluteRegionalTempPreIndustrial+s.regtmp[t,r])/absoluteRegionalTempPreIndustrial,s.diayldnl)-1.0);
    end
    else
    begin
      s.diadead[t,r]:=0.0;
      s.diasick[t,r]:=0.0;
    end;
  end;
end;

end.