// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund.Components.ImpactWaterResources;

interface

uses Esmf,
     Fund.CommonDimensions;

type

  IImpactWaterResourcesState = public interface
    property watech: IVariable1Dimensional<Timestep, double> read;
    property watechrate: Double read;

    property water: IVariable2Dimensional<Timestep,Region,double> read;
    property wrbm: IParameter1Dimensional<Region,double> read;
    property wrel: Double read;
    property wrnl: Double read;

    property gdp90: IParameter1Dimensional<Region,double> read;
    property income: IParameter2Dimensional<Timestep, Region, double> read;

    property temp: IParameter1Dimensional<Timestep, double> read;    
  end;

  ImpactWaterResourcesComponent = public class
  private
  protected
  public
    method Run(clock: Clock; state: IImpactWaterResourcesState);        
  end;
  
implementation

method ImpactWaterResourcesComponent.Run(clock: Clock; state: IImpactWaterResourcesState);
begin
  var s:=state;
  var t:=clock.Current;

  if t>Timestep.FromYear(2000) then
    s.watech[t]:=Math.Pow(1.0-s.watechrate,t.Value-Timestep.FromYear(2000).Value)
  else
    s.watech[t]:=1.0;

  for each r: Region in Enum.GetValues(typeof(Region)) do
  begin  
    var water:=s.wrbm[r]*s.gdp90[r]*s.watech[t]*Math.Pow(s.income[t,r]/s.gdp90[r],s.wrel)*Math.Pow(s.temp[t],s.wrnl);

    if water > 0.1*s.income[t,r] then
      s.water[t,r]:=0.1*s.income[t,r]
    else
      s.water[t,r]:=water;   
  end;  
end;

end.