// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund.Components.SocioEconomic;

interface

uses Esmf,
  Fund.CommonDimensions;

type
     
  ISocioEconomicState = public interface
    property population: IVariable2Dimensional<Timestep, Region, double> read;
    // Same as population just not in millions but ones
    property populationin1: IVariable2Dimensional<Timestep, Region, double> read;
    property globalpopulation: IVariable1Dimensional<Timestep, double> read;
    property pgrowth: IParameter2Dimensional<Timestep, Region, double> read;
    property enter: IParameter2Dimensional<Timestep, Region, double> read;
    property leave: IParameter2Dimensional<Timestep, Region, double> read;
    property dead: IParameter2Dimensional<Timestep, Region, double> read;

    property income: IVariable2Dimensional<Timestep, Region, double> read;
    property ypcgrowth: IParameter2Dimensional<Timestep, Region, double> read;
    property eloss: IParameter2Dimensional<Timestep, Region, double> read;
    property sloss: IParameter2Dimensional<Timestep, Region, double> read;
    property ch4cost: IVariable2Dimensional<Timestep, Region, double> read;
    property n2ocost: IVariable2Dimensional<Timestep, Region, double> read;
    property formp: IParameter1Dimensional<Region, double> read;
    property forcost: IParameter2Dimensional<Timestep, Region, double> read;
    property gdp0: IParameter1Dimensional<Region, double> read;
    
    property consumption: IVariable2Dimensional<Timestep, Region, double> read;
    property globalconsumption: IVariable1Dimensional<Timestep, double> read;
    
    property runwithoutdamage: Boolean read;
    property runwithoutpopulationperturbation: boolean read;
        
    property consleak: double read;

    property taxmp: IParameter1Dimensional<Region, double> read;
    
    // reduction inc growth taxing
    property ryg: IVariable2Dimensional<Timestep, Region, double> read;
    
    property energint: IVariable2Dimensional<Timestep, Region, double> read;
    property emissint: IVariable2Dimensional<Timestep, Region, double> read;
    
    property emission: IVariable2Dimensional<Timestep, Region, double> read;
    property emissionwithforestry: IVariable2Dimensional<Timestep,Region, double> read;
    property forestemm: IParameter2Dimensional<Timestep, Region, double> read;
    property forestry: IParameter2Dimensional<Timestep, Region, double> read;
    
    property so2WithSeeiAndScei: IVariable2Dimensional<Timestep,Region,double> read;
    property so2: IVariable2Dimensional<Timestep, Region, double> read;
    property sf6: IVariable2Dimensional<Timestep, Region, double> read;
    
    property aeei: IParameter2Dimensional<Timestep, Region, double> read;
    property reei: IVariable2Dimensional<Timestep, Region, double> read;
    property acei: IParameter2Dimensional<Timestep, Region, double> read;
    
    // reduction carbon efficiency
    property rcei: IVariable2Dimensional<Timestep, Region, double> read; 
    
    property so2pop: double read;
    property so2inc: double read;
    property so2carb: double read;
    
    property sf60: IParameter1Dimensional<Region, double> read;
    property sf6gdp: double read;
    property GDP90: IParameter1Dimensional<Region, double> read;
    property pop90: IParameter1Dimensional<Region, double> read;
    property sf6ypc: double read;
    
    property energuse: IVariable2Dimensional<Timestep, Region, double> read;
    property seei: IVariable2Dimensional<Timestep, Region, double> read;
    property scei: IVariable2Dimensional<Timestep, Region, double> read;

    property ch4: IVariable2Dimensional<Timestep, Region, double> read;
    property n2o: IVariable2Dimensional<Timestep, Region, double> read;

    property ch4em: IParameter2Dimensional<Timestep, Region, double> read;
    property n2oem: IParameter2Dimensional<Timestep, Region, double> read;
    property ch4red: IVariable2Dimensional<Timestep, Region, double> read;
    property n2ored: IVariable2Dimensional<Timestep, Region, double> read;
    
    property currtax: IParameter2Dimensional<Timestep, Region, double> read;
    property taxpar: IVariable2Dimensional<Timestep, Region, double> read;
    property minint: IVariable1Dimensional<Timestep, double> read;
    property co2red: IVariable2Dimensional<Timestep, Region, double> read;
    property know: IVariable2Dimensional<Timestep, Region, double> read;
    property globknow: IVariable1Dimensional<Timestep, double> read;
    property perm: IVariable2Dimensional<Timestep, Region, double> read;
    property knowpar: double read;
    property knowgpar: double read;
    property gwpch4: double read;
    property ch4par1: IParameter1Dimensional<Region, double> read;
    property ch4par2: IParameter1Dimensional<Region, double> read;
    property gwpn2o: double read;
    property n2opar1: IParameter1Dimensional<Region, double> read;
    property n2opar2: IParameter1Dimensional<Region, double> read;    
  end;
  
implementation

end.