// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund.Components.ValueOfStatisticalLife;

interface

uses Esmf,
     Fund.CommonDimensions;

type

  IValueOfStatisticalLifeState = public interface
    property vsl: IVariable2Dimensional<Timestep, Region, double> read;

    property vslbm: double read;
    property income: IParameter2Dimensional<Timestep, Region, double> read;
    property population: IParameter2Dimensional<Timestep, Region, double> read;
  end;

  ValueOfStatisticalLifeComponent = public class
  private
  protected
  public
    method Run(clock: Clock; state: IValueOfStatisticalLifeState);                
  end;
  
implementation

method ValueOfStatisticalLifeComponent.Run(clock: Clock; state: IValueOfStatisticalLifeState);
begin
  var s:=state;
  var t:=clock.Current;
  
  for each r: Region in Enum.GetValues(typeof(Region)) do
  begin
    s.vsl[t,r]:=s.vslbm*s.income[t,r]/s.population[t,r];
  end;
end;

end.