// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Text;

namespace Esmf
{
    public class Clock
    {
        private Timestep _startTime;
        private Timestep? _stopTime;
        private Timestep _current;

        public Clock(Timestep startTime, Timestep? stopTime)
        {
            _startTime = startTime;
            _stopTime = stopTime;
            _current = startTime;
        }

        public Timestep StartTime
        {
            get { return _startTime; }
        }

        public Timestep? StopTime
        {
            get { return _stopTime; }
        }

        public Timestep Current
        {
            get 
            {
                if (!_stopTime.HasValue)
                {
                    if (IsDone)
                    {
                        throw new InvalidOperationException("Cannot determine Current when clock is done");
                    }
                }
                return _current;
            }
        }

        public bool IsDone
        {
            get
            {
                if (!_stopTime.HasValue)
                {
                    throw new InvalidOperationException("Cannot determine IsDone for clocks witout a StopTime");
                }

                return _current > _stopTime.Value;
            }
        }

        public bool IsLastTimestep
        {
            get
            {
                if (!_stopTime.HasValue)
                {
                    throw new InvalidOperationException("Cannot determine IsLastTimestep for clocks witout a StopTime");
                }

                return _current == _stopTime.Value;
            }
        }

        public void Advance()
        {
            if (_stopTime.HasValue)
            {
                if (IsDone)
                {
                    throw new InvalidOperationException("Cannot advance clock beyond StopTime");
                }
            }
            _current=Timestep.FromSimulationYear(_current.Value+1);
        }
    }
}
