// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.Reflection.Emit;

namespace Esmf
{
    public class Variable2DimensionalTime<D2, T> : IVariable2Dimensional<Timestep, D2, T>, IParameter2Dimensional<Timestep, D2, T>
    {
        private T[,] _values;
        private Enum2IntConversionDelegate<D2> _enumConversionDelegate;
        private ModelFields _parent;

        public Variable2DimensionalTime(ModelFields parent)
        {
            // TODO Change this to properly investigate the dimensions
            _values = new T[350, 16];

            _enumConversionDelegate = Enum2IntConversionSingleton<D2>.Instance.Delegate;

            _parent = parent;
        }

        public T this[Timestep index1, D2 index2]
        {
            get
            {
                return _values[index1.Value, _enumConversionDelegate(index2) - 1];
            }
            set
            {
                _values[index1.Value, _enumConversionDelegate(index2) - 1] = value;
            }
        }


        #region IVariable2Dimensional<Timestep,D2,T> Members

        T IVariable2Dimensional<Timestep, D2, T>.this[Timestep index1, D2 index2]
        {
            get
            {
                return this[index1, index2];
            }
            set
            {
                this[index1, index2] = value;
            }
        }

        #endregion

        #region IParameter2Dimensional<Timestep,D2,T> Members

        T IParameter2Dimensional<Timestep, D2, T>.this[Timestep index1, D2 index2]
        {
            get
            {
                return this[index1, index2];
            }
        }

        IEnumerable<Parameter2DimensionalMember<Timestep, D2, T>> IParameter2Dimensional<Timestep, D2, T>.GetEnumerator()
        {
            D2[] values = (D2[]) Enum.GetValues(typeof(D2));
            for (int i = 0; i < 350; i++)
            {
                for (int l = 0; l < 16; l++)
                {
                    yield return new Parameter2DimensionalMember<Timestep,D2,T>(Timestep.FromSimulationYear(i), values[l],_values[i,l], _parent.AttachedDimensions);
                }
            }
        }

        #endregion
    }
}
