// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

uses Esmf;

type
  
  ISocioEconomic2Welfare = public interface
    property population: IParameter2Dimensional<Timestep, Fund.CommonDimensions.Region, double> read;
    property consumption: IParameter2Dimensional<Timestep, Fund.CommonDimensions.Region, double> read;
    
    property totalpopulation: IVariable1Dimensional<Timestep, double> read;
    property totalconsumption: IVariable1Dimensional<Timestep, double> read;        
  end; 
   
  SocioEconomic2Welfare = public class
  private
  protected
  public
    method Run(clock: Clock; state: ISocioEconomic2Welfare);
  end;
  
implementation

method SocioEconomic2Welfare.Run(clock: Clock; state: ISocioEconomic2Welfare); 
begin
  var s:=state;
  var t:=clock.Current;
    
  var totalpopulation:=0.0;
  var totalconsumption:=0.0;
  
  for each r: Fund.CommonDimensions.Region in Enum.GetValues(typeof(Fund.CommonDimensions.Region)) do
  begin
    totalpopulation:=totalpopulation + s.population[t,r];
    totalconsumption:=totalconsumption + s.consumption[t,r];
  end;
  
  s.totalpopulation[t]:=totalpopulation*1000000;
  s.totalconsumption[t]:=totalconsumption;
end;

end.