// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;
// Last changed: 17 July 2008

interface

type
  
  LegacyFund28Data = public class
  private
    _parameters: FundParameter;
    globemm, globcum: double;
    emmrec, count: regarr;
    
    method SetMeasures1;
    method SetFor;
    method SetInflFor(VAR country: regrec; t: Integer);
  public
    Economy: array [1..LegacyConstants.NoReg] of RegRec;
    Climate: ClimateClass;        
    scengmt: boolean := false;                 
    
    constructor(parameters: FundParameter);
    method Calculate(VAR lE: regrec; t: integer; Output: ModelOutput);
  end;

implementation

uses
  System.Threading,
  System.Globalization,
  System.IO,
  System.Xml,
  System.Xml.XPath,
  System.Reflection,
  System.Configuration,
  System.Collections.Specialized,
  System.Diagnostics;
     
constructor LegacyFund28Data(parameters: FundParameter); 
begin
  _parameters:=parameters;
  Climate:=new ClimateClass;
  
  for i: integer :=1 to LegacyConstants.NoReg do
  begin
    economy[i]:=new RegRec();
    Economy[i].regionIndex:=i;
  end;
     
     economy[1].cummemm:=  22967;        economy[2].cummemm:=   1999;
     economy[3].cummemm:=  25854;        economy[4].cummemm:=   1135;
     economy[5].cummemm:=    526;        economy[6].cummemm:=   2199;
     economy[7].cummemm:=   3214;        economy[8].cummemm:=     86;
     economy[9].cummemm:=    212;        economy[10].cummemm:=   696;
     economy[11].cummemm:=   569;        economy[12].cummemm:=   225;
     economy[13].cummemm:=   775;        economy[14].cummemm:=    64;
     economy[15].cummemm:=   472;        economy[16].cummemm:=     0;

     Economy[1].area:=   9207581;       Economy[2].area:=   9872483;
     Economy[3].area:=   3705040;       Economy[4].area:=    472934;
     Economy[5].area:=   9235211;       Economy[6].area:=   2186128;
     Economy[7].area:=  21716915;       Economy[8].area:=   6313671;
     Economy[9].area:=   2440575;       Economy[10].area:= 17451759;
     Economy[11].area:=  5137567;       Economy[12].area:=  4958546;
     Economy[13].area:= 10958008;       Economy[14].area:=  3953026;
     Economy[15].area:= 24329621;       Economy[16].area:=   332534;

     globcum:= 60994.3;

     SetMeasures1;
  for i: Integer := 1 to LegacyConstants.NoReg do
  begin
    economy[i].popdens:= parameters.pop0[i]/economy[i].area*1000000;
    // Debug.Assert(popdens>=0, 'popdens<0');
    economy[i].YpC:= 1000*parameters.gdp0[i]/_parameters.pop0[i];
    economy[i].urbpop:= (0.031*LegacyFund28Math.sqrt(economy[i].ypc)-0.011*LegacyFund28Math.sqrt(economy[i].popdens))/(1+0.031*LegacyFund28Math.sqrt(economy[i].ypc)-0.011*LegacyFund28Math.sqrt(economy[i].popdens));
    economy[i].emission:= parameters.emissint0[i]*_parameters.gdp0[i];
    economy[i].emiss0:= economy[i].emission;                        (* confirmed *)              
    economy[i].wetmeas:= 0;
    economy[i].dryland:= 0;
    economy[i].drycost:= 0;
    economy[i].wetcost:= 0;
    economy[i].leave:= 0;
    economy[i].cumleave:= 0;
    economy[i].cumenter:= 0;
    economy[i].entercost:= 0;
    economy[i].leavecost:= 0;
    economy[i].cumaeei:=1;
  end;
     
  SetMeasures1     
END;

(* FORESTRY PROCEDURES *)

method LegacyFund28Data.SetInflFor(VAR country: regrec; t: Integer);
(* determines the influence of COUNTRY.FORESTRY on the economy of COUNTRY *)

VAR fca, fcd, fa, fd, th1, th2: double;

BEGIN
          IF country.forestry[t] > 0 THEN BEGIN
             th1:= _parameters.forconst[country.RegionIndex] - _parameters.forlinear[country.RegionIndex] + LegacyFund28Math.sqrt(LegacyFund28Math.sqr(_parameters.forconst[country.RegionIndex]-_parameters.forlinear[country.RegionIndex])-4*_parameters.forconst[country.RegionIndex]*_parameters.forsquare[country.RegionIndex]);
             th1:= th1/2/_parameters.forsquare[country.RegionIndex];
             th2:= (3*_parameters.forconst[country.RegionIndex]/2 - _parameters.forlinear[country.RegionIndex])/2/_parameters.forsquare[country.RegionIndex];
             IF country.forestry[t] < (th1 + th2) THEN BEGIN
                fd:= country.forestry[t];
                fa:= 0
             END
             ELSE IF country.forestry[t] < (2*th1 + th2) THEN BEGIN
                  fd:= th1 + th2;
                  fa:= country.forestry[t] - fd
             END
             ELSE BEGIN
                  fa:= th1;
                  fd:= country.forestry[t] - fa
             END;
             IF fd > _parameters.forestemm[t, country.regionIndex] THEN BEGIN    (* !!! *)
                fd:= _parameters.forestemm[t, country.regionIndex];
                fa:= country.forestry[t] - fd
             END;
             IF fd > 0 THEN BEGIN
                fcd:= 2*(_parameters.forconst[country.RegionIndex] + _parameters.forlinear[country.RegionIndex]*fd + _parameters.forsquare[country.RegionIndex]*LegacyFund28Math.sqr(fd))/3;
                IF fcd < _parameters.forconst[country.RegionIndex]*fd THEN
                   fcd:= 2*_parameters.forconst[country.RegionIndex]*fd/3
             END
             ELSE
                 fcd:= 0;
             IF fa > 0 THEN BEGIN
                fca:= _parameters.forconst[country.RegionIndex] + _parameters.forlinear[country.RegionIndex]*fa + _parameters.forsquare[country.RegionIndex]*LegacyFund28Math.sqr(fa);
                IF fca < _parameters.forconst[country.RegionIndex]*fa THEN
                   fca:= _parameters.forconst[country.RegionIndex]*fa
             END
             ELSE
                 fca:= 0;
             country.forcost:= fca + fcd
          END
          ELSE
              country.forcost:= 0
END;

method LegacyFund28Data.SetFor;
(* sets all forestry to zero *)

VAR i, j: Integer;

BEGIN
     FOR i:= 1 TO LegacyConstants.NoReg DO
         FOR j:= 1 TO LegacyConstants.NYear DO
             economy[i].forestry[j]:= 0;
END;

(* SETTINGS *)

PROCEDURE LegacyFund28Data.SetMeasures1;
(* sets all measures to zero *)

BEGIN
     SetFor
END;

(* CALCULATING PROCEDURES *)
// Output:  Calculate writes a number of model results into the object referenced
//          by that variable. Pass in nil if that option is not wanted.
method LegacyFund28Data.Calculate(VAR lE: regrec; t: integer; Output: ModelOutput);
VAR
  j: integer;
  ygrowth: double;
  lC: ClimateClass;
  lR: Integer;
  lP: FundParameter;
BEGIN
  lC:=Climate;
  lR:=lE.regionIndex;
  lP:=_parameters;
  
  ygrowth:=lE.income/lE.oldincome - 1.0;
    
          IF lE.population < 0.000000001 THEN
             lE.YpC:= 0
          ELSE
              lE.YpC:= 1000*lE.income/lE.population;
          
(* MITIGATION *)
          

          setinflfor(var lE,t);

(* CLIMATE IMPACTS *)
          // TODO -oDavid Anthoff: RT uses -lP.forel for ppp case
          lE.forests:= lP.forbm[lR]*lE.income*LegacyFund28Math.pow(lE.ypc/lP.ypc90[lR],lP.forel)*(0.5*LegacyFund28Math.pow(lE.regtmp/lP.bregtmp[lR],lP.fornl)+0.5*LegacyFund28Math.ln(lC.acco2[t-1]/lP.co2pre)*lP.forco2);
          IF lE.forests > 0.1*lE.income THEN
            lE.forests:= 0.1*lE.income;
           
          IF t > 50 THEN
            lE.cumaeei:= lE.cumaeei*(1-0.01*(lP.aeei[t,lR]+lP.aeeiadd[lR]*(t-50))-lE.reei);

          lE.dengue:= lP.dfbs[lR]*lE.population*lP.dfch[lR]*LegacyFund28Math.pow(lE.regtmp/lP.bregtmp[lR],lP.dfnl)*LegacyFund28Math.pow(lE.ypc/lP.ypc90[lR],lP.vbel);

          lE.schisto:= lP.smbs[lR]*lE.population*lP.smch[lR]*LegacyFund28Math.pow(lE.regtmp/lP.bregtmp[lR],lP.smnl)*LegacyFund28Math.pow(lE.ypc/lP.ypc90[lR],lP.vbel);
          IF lE.schisto < -lP.smbs[lR]*lE.population*LegacyFund28Math.pow(lE.ypc/lP.ypc90[lR],lP.vbel) THEN
            lE.schisto:= -lP.smbs[lR]*lE.population*LegacyFund28Math.pow(lE.ypc/lP.ypc90[lR],lP.vbel);

          lE.malaria:= lP.malbs[lR]*lE.population*lP.malch[lR]*LegacyFund28Math.pow(lE.regtmp/lP.bregtmp[lR],lP.malnl)*LegacyFund28Math.pow(lE.ypc/lP.ypc90[lR],lP.vbel);


          lE.plus:= lP.plus90[lR]*LegacyFund28Math.pow(lE.ypc/lP.ypc90[lR],lP.plusel);
          IF lE.plus > 1 THEN
            lE.plus:= 1;
          lE.urbpop:= (0.031*LegacyFund28Math.sqrt(lE.ypc)-0.011*LegacyFund28Math.sqrt(lE.popdens))/(1+0.031*LegacyFund28Math.sqrt(lE.ypc)-0.011*LegacyFund28Math.sqrt(lE.popdens))
                   /(1+lP.urbcorr[lR]/(1+0.001*LegacyFund28Math.pow(t-40,2))); (* DA: urbcorr needs to be changed to a function if this is to be made uncertain *)
          lE.basecardvasc:= lP.cardvasc90[lR] + lP.cvlin*(lE.plus-lP.plus90[lR]);
          IF lE.basecardvasc > 1 THEN
            lE.basecardvasc:= 1;
          lE.baseresp:= lP.resp90[lR] + lP.rlin*(lE.plus-lP.plus90[lR]);
          IF lE.baseresp > 1 THEN
            lE.baseresp:= 1;

          lE.cardheat:= (lP.chplbm[lR]*lE.plus + lP.chmlbm[lR]*(1-lE.plus))*lE.regtmp/lP.bregtmp[lR] +
                     (lP.chpqbm[lR]*lE.plus + lP.chmqbm[lR]*(1-lE.plus))*LegacyFund28Math.pow(lE.regtmp/lP.bregtmp[lR],2);
          lE.cardheat:= lE.cardheat*lE.urbpop*lE.population*10;
          IF lE.cardheat > 1000*lP.maxcardvasc*lE.basecardvasc*lE.urbpop*lE.population THEN
             lE.cardheat:= 1000*lP.maxcardvasc*lE.basecardvasc*lE.urbpop*lE.population;
          IF lE.cardheat < 0 THEN
             lE.cardheat:= 0;

          lE.hurrdam:= 0.0001*lP.hurrbasedam[lR]*lE.income*LegacyFund28Math.pow(lE.ypc/lP.ypc90[lR],lP.hurrdamel)*(LegacyFund28Math.pow(1+lP.hurrpar*lE.regtmp,lP.hurrnl)-1);
          lE.hurrdead:= 1000*lP.hurrbasedead[lR]*lE.population*LegacyFund28Math.pow(lE.ypc/lP.ypc90[lR],lP.hurrdeadel)*(LegacyFund28Math.pow(1+lP.hurrpar*lE.regtmp,lP.hurrnl)-1);
          lE.resp:= lP.rlbm[lR]*lE.regtmp/lP.bregtmp[lR] + lP.rqbm[lR]*LegacyFund28Math.pow(lE.regtmp/lP.bregtmp[lR],2);
          lE.resp:= lE.resp*lE.urbpop*lE.population*10;
          IF lE.resp > 1000*lP.maxcardvasc*lE.baseresp*lE.urbpop*lE.population THEN
             lE.resp:= 1000*lP.maxcardvasc*lE.baseresp*lE.urbpop*lE.population;
          IF lE.resp < 0 THEN
             lE.resp:= 0;
          
          lE.cardcold:= (lP.ccplbm[lR]*lE.plus + lP.ccmlbm[lR]*(1-lE.plus))*lE.regtmp/lP.bregtmp[lR] +
                     (lP.ccpqbm[lR]*lE.plus + lP.ccmqbm[lR]*(1-lE.plus))*LegacyFund28Math.pow(lE.regtmp/lP.bregtmp[lR],2);
          lE.cardcold:= lE.cardcold*lE.population*10;
          IF lE.cardcold < -1000*lP.maxcardvasc*lE.basecardvasc*lE.population THEN
             lE.cardcold:= -1000*lP.maxcardvasc*lE.basecardvasc*lE.population;
          IF lE.cardcold > 0 THEN
             lE.cardcold:= 0;

          lE.dead:= lE.dengue + lE.schisto + lE.malaria + lE.cardheat + lE.cardcold + lE.resp + lE.diadead + lE.hurrdead;
          IF (lE.dead > lE.population*1000000) THEN
            lE.dead:= lE.population/1000000;
          lE.yll:= lP.d2ld[lR]*lE.dengue+lP.d2ls[lR]*lE.schisto+lP.d2lm[lR]*lE.malaria+lP.d2lc[lR]*lE.cardheat+lP.d2lc[lR]*lE.cardcold+lP.d2lr[lR]*lE.resp; (* DA: no param, but data from WHO; therefor no uncertainty *)
          lE.yld:= lP.d2dd[lR]*lE.dengue+lP.d2ds[lR]*lE.schisto+lP.d2dm[lR]*lE.malaria+lP.d2dc[lR]*lE.cardheat+lP.d2dc[lR]*lE.cardcold+lP.d2dr[lR]*lE.resp+lE.diasick;
          lE.deadcost:= lP.vsl*lE.ypc*lE.dead/1000000000;
          (*deadcost:= vyll*ypc*yll/1000000000;*)
          lE.morbcost:= lP.vmorb*lE.ypc*lE.yld/1000000000;

          lE.wetval:= lP.wvbm[lR]/lP.valbase*lE.ypc/lP.valinc[lR]/(1+lE.ypc/lP.valinc[lR])/lP.valadj[lR]*lE.ypc;
          IF (lP.wetmax[lR] > 0) THEN
             IF lE.wettot > lP.wetmax[lR]*(1-lP.bioshare/(1+lP.bioshare)) THEN
                lE.wetval:= lE.wetval*2
             ELSE
                 lE.wetval:= lE.wetval*((1-lP.bioshare)+lP.bioshare*lP.wetmax[lR]/(lP.wetmax[lR]-lE.wettot));
          lE.dryval:= lP.dvbm[lR]*lE.popdens*lE.ypc/1000/lP.incdens;
          lE.npprotcost:= lP.pc[lR]*lC.ds*100*(1+lP.discrate[lR]+ygrowth)/(lP.discrate[lR]+ygrowth);
          IF lE.npprotcost < 0 THEN
             lE.npprotcost:= 0;
          lE.npwetcost:= lE.wetval*lP.wmbm[lR]*lC.ds/(lP.discrate[lR]+ygrowth);
          IF lE.npwetcost < 0 THEN
             lE.npwetcost:= 0;
          lE.npdrycost:= lE.dryval*lP.dlbm[lR]*lC.ds*(1+ygrowth)*(lP.discrate[lR]+ygrowth)/lP.discrate[lR]/lP.discrate[lR];
          IF lE.npdrycost < 0.00000001 THEN BEGIN
             lE.npdrycost:= 0;
             lE.protlev:= 0
          END
          ELSE
              lE.protlev:= 1-0.5*(lE.npprotcost+lE.npwetcost)/lE.npdrycost;
          IF lE.protlev < 0 THEN
             lE.protlev:= 0;
          lE.wetland:= lP.wlbm[lR]*lC.sea; (* DA: sea = sea level rise in meters since 1750, wlbm = loss of land per meter of sea level rise in square km *)
          if lC.ds > 0 then
            lE.wetmeas:= lP.wmbm[lR]*lC.ds*lE.protlev + lE.wetmeas;

          lE.wettot:= lE.wetland + lE.wetmeas;
          IF lC.ds > 0 THEN
            lE.wetann:= (lP.wlbm[lR] + lP.wmbm[lR]*lE.protlev)*lC.ds
          ELSE
            lE.wetann:=0;

          IF lE.wettot > lP.wetmax[lR] THEN BEGIN
             lE.wettot:= lP.wetmax[lR];
             lE.wetann:= 0
          END;
          lE.wetcost:= lE.wetval*lE.wetann/1000 + lP.wetdel[lR]*lE.wetcost;
          if lC.ds > 0 then
          begin
            lE.dryland:= lP.dlbm[lR]*lC.ds*(1-lE.protlev) + lE.dryland; (* DA: ds = change of sea level in current year, assumption: income is always increasing -> hardwired *)
            lE.area:= lE.area - lP.dlbm[lR]*lC.ds*(1-lE.protlev);
            lE.drycost:= lE.dryval*lP.dlbm[lR]*lC.ds*(1-lE.protlev)/1000 + lP.drydel[lR]*lE.drycost;
          end;

          IF lE.drycost < 0 THEN
            lE.drycost:= 0;
          lE.protcost:= lE.protlev*lP.pc[lR]*lC.ds/10;
          IF lE.protcost < 0 THEN
            lE.protcost:= 0;
          lE.enter:= 0;
          FOR j:= 1 TO LegacyConstants.NoReg DO
          begin
              lE.enter:= lE.enter + lP.immigrate[lR, j]*economy[j].leave;
          end;
          lE.leave:= lP.coastpd[lR]*lE.popdens*lP.dlbm[lR]*lC.ds*(1-lE.protlev);
          IF lE.leave < 0 THEN
            lE.leave:= 0;
          Debug.Assert(lE.leave>=0, 'leave<0');
          lE.cumleave:= lE.leave + lE.cumleave;
          lE.cumenter:= lE.enter + lE.cumenter;
          lE.leavecost:= lP.emcst*lE.ypc*lE.leave/1000000000 + lP.leavedel*lE.leavecost;
          IF lP.entertime < 1 THEN
            lP.entertime:= 1;
          lE.entercost:= lP.immcst*lE.ypc*lE.enter/1000000000 + (1-1/lP.entertime)*lE.entercost;

          lE.popdens:= lE.population/lE.area*1000000;

          // Some modes won't pass in an Output object, in that
          // case don't write any damages to the Output object
          if Output<>nil then
          begin
            // Write each damage seperately into the Output object
            // Every value is multiplied by 1 bn, since the damage
            // figures in the individual variables like
            // water, forests etc. are in bn $
            Output.Damages.Add(new Damage(t, lR, Sector.sWater, -lE.water*1000000000));
            Output.Damages.Add(new Damage(t, lR, Sector.sForests, -lE.forests*1000000000));
            Output.Damages.Add(new Damage(t, lR, Sector.sHeating, -lE.heating*1000000000));
            Output.Damages.Add(new Damage(t, lR, Sector.sCooling, -lE.cooling*1000000000));
            Output.Damages.Add(new Damage(t, lR, Sector.sAgriculture, -lE.agcost*1000000000));
            Output.Damages.Add(new Damage(t, lR, Sector.sDryland, lE.drycost*1000000000));
            Output.Damages.Add(new Damage(t, lR, Sector.sSeaprotection, lE.protcost*1000000000));
            Output.Damages.Add(new Damage(t, lR, Sector.sImigration, lE.entercost*1000000000));
            Output.Damages.Add(new Damage(t, lR, Sector.sHurrican, lE.hurrdam*1000000000));

            Output.Damages.Add(new Damage(t, lR, Sector.sSpecies, lE.species*1000000000));
            Output.Damages.Add(new Damage(t, lR, Sector.sDeath, lE.deadcost*1000000000));
            Output.Damages.Add(new Damage(t, lR, Sector.sMorbidity, lE.morbcost*1000000000));
            Output.Damages.Add(new Damage(t, lR, Sector.sWetland, lE.wetcost*1000000000));
            Output.Damages.Add(new Damage(t, lR, Sector.sEmigration, lE.leavecost*1000000000));
          end;
         
END;

END.
