// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

type
  AverageEquityWeightingFirstPeriod = public class( WeightingScheme )
  protected
    function GetName: string; override;
  public
    procedure CalculateWeights(ModelOutput: ModelOutput); override;
  end;
  
implementation

procedure AverageEquityWeightingFirstPeriod.CalculateWeights(
  ModelOutput: ModelOutput);
var
  i_worldIncome, i_worldPopulation: double;
  i_equityWeight: double;
  i, l: Integer;
begin
  inherited;

  i_worldIncome:=0;
  i_worldPopulation:=0;

  for l:=1 to LegacyConstants.NoReg do
  begin
    i_worldIncome:=i_worldIncome + ModelOutput.Incomes.Item[50, l];
    i_worldPopulation:=i_worldPopulation + ModelOutput.Populations.Item[50, l];
  end;

  // TODO -oDavid Anthoff: Change start of "real" period to constant
  for i:=50 to LegacyConstants.NYear do
  begin
    for l:=1 to LegacyConstants.NoReg do
    begin
      i_equityWeight:=(i_worldIncome/i_worldPopulation)/(ModelOutput.Incomes.Item[50, l]/ModelOutput.Populations.Item[50, l]);
      AddWeight(i, l, i_equityWeight);
    end;
  end;
end;

function AverageEquityWeightingFirstPeriod.GetName: string;
begin
  Result:='Average equity weighting first period';
end;

end.