// Climate Framework for Uncertainty, Negotiation and Distribution
// Version 3.3
// (c) David Anthoff and Richard S.J. Tol
// http://www.fund-model.org
namespace Fund;

interface

type
  HeterogeneousPRTP = public class( WeightingScheme )
  private
    m_Mju: double;
  protected
    function GetName: string; override;
  public
    constructor(Mju: double);

    procedure CalculateWeights(ModelOutput: ModelOutput); override;
  end;
  
implementation

{ HeterogeneousPRTP2 }

procedure HeterogeneousPRTP.CalculateWeights(ModelOutput: ModelOutput);
var
  i_prtpFactor, i_prtp: double;
  i, l: integer;
  i_newPerCapitaIncome, i_oldPerCapitaIncome: double;
  i_perCapitaGrowthRate: double;
  i_discountFactor: double;
begin
  for i:=1 to LegacyConstants.NoReg do
  begin
    i_discountFactor:=1;
    i_prtpFactor:=1;
    // TODO -oDavid Anthoff: Change start of "real" period to constant
    for l:=50 to LegacyConstants.NYear do
    begin
      AddWeight(l, i, i_discountFactor);
      i_newPerCapitaIncome:=ModelOutput.Incomes.Item[l, i]/ModelOutput.Populations.Item[l, i];
      i_oldPerCapitaIncome:=ModelOutput.Incomes.Item[l-1, i]/ModelOutput.Populations.Item[l-1, i];
      i_perCapitaGrowthRate:=(i_newPerCapitaIncome-i_oldPerCapitaIncome)/i_oldPerCapitaIncome;

      i_prtp:=2/((l-50)+1/m_Mju);

      i_discountFactor:=i_discountFactor/(1.00 + i_prtp + i_perCapitaGrowthRate);
    end;
  end;
end;

constructor HeterogeneousPRTP(Mju: double);
begin
  inherited constructor;
  m_Mju:=Mju;
end;

function HeterogeneousPRTP.GetName: string;
begin
  Result:='Heterogeneous prtp (mju=' + Convert.ToString(m_Mju*100) + '%)';
end;

end.